@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2019 Oracle Corporation
+ * Copyright (C) 2011-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,22 +23,11 @@
 
 #include "VBoxDispD3DIf.h"
 #include "../../common/wddm/VBoxMPIf.h"
-#ifdef VBOX_WITH_CRHGSMI
-#include "VBoxUhgsmiDisp.h"
-#endif
-
-#ifdef VBOX_WDDMDISP_WITH_PROFILE
-#include <iprt/asm.h>
-extern volatile uint32_t g_u32VBoxDispProfileFunctionLoggerIndex;
-# define VBOXDISPPROFILE_FUNCTION_LOGGER_INDEX_GEN() ASMAtomicIncU32(&g_u32VBoxDispProfileFunctionLoggerIndex);
-# include "VBoxDispProfile.h"
-#endif
 
 #include <iprt/cdefs.h>
 #include <iprt/list.h>
 
 #define VBOXWDDMDISP_MAX_VERTEX_STREAMS 16
-#define VBOXWDDMDISP_MAX_SWAPCHAIN_SIZE 16
 #define VBOXWDDMDISP_MAX_TEX_SAMPLERS 16
 #define VBOXWDDMDISP_TOTAL_SAMPLERS VBOXWDDMDISP_MAX_TEX_SAMPLERS + 5
 #define VBOXWDDMDISP_SAMPLER_IDX_IS_SPECIAL(_i) ((_i) >= D3DDMAPSAMPLER && (_i) <= D3DVERTEXTEXTURESAMPLER3)
@@ -89,10 +78,6 @@ typedef struct VBOXWDDMDISP_ADAPTER
 
     VBOXWDDM_QAI AdapterInfo;
 
-#ifdef VBOX_WDDMDISP_WITH_PROFILE
-    VBoxDispProfileFpsCounter ProfileDdiFps;
-    VBoxDispProfileSet ProfileDdiFunc;
-#endif
 #ifdef VBOX_WITH_VIDEOHWACCEL
     uint32_t cHeads;
     VBOXWDDMDISP_HEAD aHeads[1];
@@ -154,42 +139,7 @@ typedef struct VBOXWDDMDISP_RENDERTGT
     VBOXWDDMDISP_RENDERTGT_FLAGS fFlags;
 } VBOXWDDMDISP_RENDERTGT, *PVBOXWDDMDISP_RENDERTGT;
 
-#define VBOXWDDMDISP_INDEX_UNDEFINED (~0U)
-typedef struct VBOXWDDMDISP_SWAPCHAIN_FLAGS
-{
-    union
-    {
-        struct
-        {
-            UINT bChanged                : 1;
-            UINT bRtReportingPresent     : 1; /* use VBox extension method for performing present */
-            UINT bSwitchReportingPresent : 1; /* switch to use VBox extension method for performing present on next present */
-            UINT Reserved                : 29;
-        };
-        uint32_t Value;
-    };
-}VBOXWDDMDISP_SWAPCHAIN_FLAGS;
-
-typedef struct VBOXWDDMDISP_SWAPCHAIN
-{
-    RTLISTNODE ListEntry;
-    UINT iBB; /* Backbuffer index */
-    UINT cRTs; /* Number of render targets in the swapchain */
-    VBOXWDDMDISP_SWAPCHAIN_FLAGS fFlags;
-#ifndef VBOXWDDM_WITH_VISIBLE_FB
-    IDirect3DSurface9 *pRenderTargetFbCopy;
-    BOOL bRTFbCopyUpToDate;
-#endif
-    IDirect3DSwapChain9 *pSwapChainIf;
-    /* a read-only hWnd we receive from wine
-     * we use it for visible region notifications only,
-     * it MUST NOT be destroyed on swapchain destruction,
-     * wine will handle that for us */
-    HWND hWnd;
-    VBOXDISP_KMHANDLE hSwapchainKm;
-    VBOXWDDMDISP_RENDERTGT aRTs[VBOXWDDMDISP_MAX_SWAPCHAIN_SIZE];
-} VBOXWDDMDISP_SWAPCHAIN, *PVBOXWDDMDISP_SWAPCHAIN;
-
+typedef struct VBOXWDDMDISP_DEVICE *PVBOXWDDMDISP_DEVICE;
 typedef HRESULT FNVBOXWDDMCREATEDIRECT3DDEVICE(PVBOXWDDMDISP_DEVICE pDevice);
 typedef FNVBOXWDDMCREATEDIRECT3DDEVICE *PFNVBOXWDDMCREATEDIRECT3DDEVICE;
 
@@ -203,7 +153,6 @@ typedef struct VBOXWDDMDISP_DEVICE
     PFNVBOXWDDMCREATEDIRECT3DDEVICE pfnCreateDirect3DDevice;
     PFNVBOXWDDMCREATESHAREDPRIMARY pfnCreateSharedPrimary;
     IDirect3DDevice9 *pDevice9If;
-    RTLISTANCHOR SwapchainList;
     UINT u32IfVersion;
     UINT uRtVersion;
     D3DDDI_DEVICECALLBACKS RtCallbacks;
@@ -217,13 +166,18 @@ typedef struct VBOXWDDMDISP_DEVICE
     struct VBOXWDDMDISP_ALLOCATION *aStreamSource[VBOXWDDMDISP_MAX_VERTEX_STREAMS];
     VBOXWDDMDISP_STREAM_SOURCE_INFO StreamSourceInfo[VBOXWDDMDISP_MAX_VERTEX_STREAMS];
     VBOXWDDMDISP_INDICES_INFO IndiciesInfo;
-    /* need to cache the ViewPort data because IDirect3DDevice9::SetViewport
-     * is split into two calls : SetViewport & SetZRange */
+    /* Need to cache the ViewPort data because IDirect3DDevice9::SetViewport
+     * is split into two calls: SetViewport & SetZRange.
+     * Also the viewport must be restored after IDirect3DDevice9::SetRenderTarget.
+     */
     D3DVIEWPORT9 ViewPort;
+    /* The scissor rectangle must be restored after IDirect3DDevice9::SetRenderTarget. */
+    RECT ScissorRect;
+    /* Whether the ViewPort field is valid, i.e. GaDdiSetViewport has been called. */
+    bool fViewPort : 1;
+    /* Whether the ScissorRect field is valid, i.e. GaDdiSetScissorRect has been called. */
+    bool fScissorRect : 1;
     VBOXWDDMDISP_CONTEXT DefaultContext;
-#ifdef VBOX_WITH_CRHGSMI
-    VBOXUHGSMI_PRIVATE_D3D Uhgsmi;
-#endif
 
     /* no lock is needed for this since we're guaranteed the per-device calls are not reentrant */
     RTLISTANCHOR DirtyAllocList;
@@ -235,17 +189,6 @@ typedef struct VBOXWDDMDISP_DEVICE
 
     HMODULE hHgsmiTransportModule;
 
-#ifdef VBOX_WDDMDISP_WITH_PROFILE
-    VBoxDispProfileFpsCounter ProfileDdiFps;
-    VBoxDispProfileSet ProfileDdiFunc;
-
-    VBoxDispProfileSet ProfileDdiPresentCb;
-#endif
-
-#ifdef VBOXWDDMDISP_DEBUG_TIMER
-    HANDLE hTimerQueue;
-#endif
-
     UINT cRTs;
     struct VBOXWDDMDISP_ALLOCATION * apRTs[1];
 } VBOXWDDMDISP_DEVICE, *PVBOXWDDMDISP_DEVICE;
@@ -298,7 +241,6 @@ typedef struct VBOXWDDMDISP_ALLOCATION
     VBOXWDDMDISP_LOCKINFO LockInfo;
     VBOXWDDM_DIRTYREGION DirtyRegion; /* <- dirty region to notify host about */
     VBOXWDDM_SURFACE_DESC SurfDesc;
-    PVBOXWDDMDISP_SWAPCHAIN pSwapchain;
 #ifdef VBOX_WITH_MESA3D
     uint32_t hostID;
 #endif
@@ -339,44 +281,6 @@ typedef struct VBOXWDDMDISP_OVERLAY
 #define VBOXDISP_CUBEMAP_INDEX_TO_FACE(pRc, idx) ((D3DCUBEMAP_FACES)(D3DCUBEMAP_FACE_POSITIVE_X+(idx)/VBOXDISP_CUBEMAP_LEVELS_COUNT(pRc)))
 #define VBOXDISP_CUBEMAP_INDEX_TO_LEVEL(pRc, idx) ((idx)%VBOXDISP_CUBEMAP_LEVELS_COUNT(pRc))
 
-DECLINLINE(PVBOXWDDMDISP_SWAPCHAIN) vboxWddmSwapchainForAlloc(PVBOXWDDMDISP_ALLOCATION pAlloc)
-{
-    return pAlloc->pSwapchain;
-}
-
-DECLINLINE(UINT) vboxWddmSwapchainIdxFb(PVBOXWDDMDISP_SWAPCHAIN pSwapchain)
-{
-    return (pSwapchain->iBB + pSwapchain->cRTs - 1) % pSwapchain->cRTs;
-}
-
-/* if swapchain contains only one surface returns this surface */
-DECLINLINE(PVBOXWDDMDISP_RENDERTGT) vboxWddmSwapchainGetBb(PVBOXWDDMDISP_SWAPCHAIN pSwapchain)
-{
-    if (pSwapchain->cRTs)
-    {
-        Assert(pSwapchain->iBB < pSwapchain->cRTs);
-        return &pSwapchain->aRTs[pSwapchain->iBB];
-    }
-    return NULL;
-}
-
-DECLINLINE(PVBOXWDDMDISP_RENDERTGT) vboxWddmSwapchainGetFb(PVBOXWDDMDISP_SWAPCHAIN pSwapchain)
-{
-    if (pSwapchain->cRTs)
-    {
-        UINT iFb = vboxWddmSwapchainIdxFb(pSwapchain);
-        return &pSwapchain->aRTs[iFb];
-    }
-    return NULL;
-}
-
 void vboxWddmResourceInit(PVBOXWDDMDISP_RESOURCE pRc, UINT cAllocs);
 
-#ifndef IN_VBOXCRHGSMI
-PVBOXWDDMDISP_SWAPCHAIN vboxWddmSwapchainFindCreate(PVBOXWDDMDISP_DEVICE pDevice, PVBOXWDDMDISP_ALLOCATION pBbAlloc, BOOL *pbNeedPresent);
-HRESULT vboxWddmSwapchainChkCreateIf(PVBOXWDDMDISP_DEVICE pDevice, PVBOXWDDMDISP_SWAPCHAIN pSwapchain);
-VOID vboxWddmSwapchainDestroy(PVBOXWDDMDISP_DEVICE pDevice, PVBOXWDDMDISP_SWAPCHAIN pSwapchain);
-
-#endif
-
 #endif /* !GA_INCLUDED_SRC_WINNT_Graphics_Video_disp_wddm_VBoxDispD3D_h */
