@@ -185,102 +185,6 @@ static int vboxConnectToVBoxTray(RTLOCALIPCSESSION *phSession)
     return rc;
 }
 
-#ifndef UNICODE
-static void vboxChar2WCharFree(PWCHAR pwString)
-{
-    if (pwString)
-        HeapFree(GetProcessHeap(), 0, pwString);
-}
-
-static int vboxChar2WCharAlloc(const TCHAR *pszString, PWCHAR *ppwString)
-{
-    int rc = VINF_SUCCESS;
-
-    int iLen = (int)_tcslen(pszString) + 2;
-    WCHAR *pwString = (WCHAR*)HeapAlloc(GetProcessHeap(), 0, iLen * sizeof(WCHAR));
-    if (!pwString)
-    {
-        rc = VERR_NO_MEMORY;
-    }
-    else
-    {
-        if (MultiByteToWideChar(CP_ACP, 0, pszString, -1, pwString, iLen) == 0)
-        {
-            rc = VERR_INVALID_PARAMETER;
-            HeapFree(GetProcessHeap(), 0, pwString);
-        }
-        else
-        {
-            *ppwString = pwString;
-        }
-    }
-    return rc;
-}
-#endif /* !UNICODE */
-
-/**
- * Disables the Windows File Protection for a specified file
- * using an undocumented SFC API call. Don't try this at home!
- *
- * @param   hwndParent          Window handle of parent.
- * @param   string_size         Size of variable string.
- * @param   variables           The actual variable string.
- * @param   stacktop            Pointer to a pointer to the current stack.
- * @param   extra               Extra parameters. Currently unused.
- */
-VBOXINSTALLHELPER_EXPORT DisableWFP(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop,
-                                    extra_parameters *extra)
-{
-    RT_NOREF(hwndParent, extra);
-
-    EXDLL_INIT();
-
-    TCHAR szFile[MAX_PATH + 1];
-    int rc = vboxPopString(szFile, sizeof(szFile) / sizeof(TCHAR));
-    if (RT_SUCCESS(rc))
-    {
-        RTLDRMOD hSFC;
-        rc = RTLdrLoadSystem("sfc_os.dll", false /* fNoUnload */, &hSFC);
-        if (RT_SUCCESS(rc))
-        {
-            PFNSFCFILEEXCEPTION pfnSfcFileException = NULL;
-            rc = RTLdrGetSymbol(hSFC, "SfcFileException", (void **)&pfnSfcFileException);
-            if (RT_FAILURE(rc))
-            {
-                /* Native fallback. */
-                HMODULE hSFCNative = (HMODULE)RTLdrGetNativeHandle(hSFC);
-
-                /* If we didn't get the proc address with the call above, try it harder with
-                 * the (zero based) index of the function list (ordinal). */
-                pfnSfcFileException = (PFNSFCFILEEXCEPTION)GetProcAddress(hSFCNative, (LPCSTR)5);
-                if (pfnSfcFileException)
-                    rc = VINF_SUCCESS;
-            }
-
-            if (RT_SUCCESS(rc))
-            {
-#ifndef UNICODE
-                WCHAR *pwszFile;
-                rc = vboxChar2WCharAlloc(szFile, &pwszFile);
-                if (RT_SUCCESS(rc))
-                {
-#else
-                    TCHAR *pwszFile = szFile;
-#endif
-                    if (pfnSfcFileException(0, pwszFile, UINT32_MAX) != 0)
-                        rc = VERR_ACCESS_DENIED; /** @todo Find a better rc. */
-#ifndef UNICODE
-                    vboxChar2WCharFree(pwszFile);
-                }
-#endif
-            }
-
-            RTLdrClose(hSFC);
-        }
-    }
-
-    vboxPushRcAsString(rc);
-}
 
 /**
  * Retrieves a file's architecture (x86 or amd64).
