@@ -28,17 +28,21 @@
 
 class VBoxDnDWnd;
 
+/**
+ * Class for implementing IDataObject for VBoxTray's DnD support.
+ */
 class VBoxDnDDataObject : public IDataObject
 {
 public:
 
     enum Status
     {
-        Uninitialized = 0,
-        Initialized,
-        Dropping,
-        Dropped,
-        Aborted
+        Status_Uninitialized = 0,
+        Status_Initialized,
+        Status_Dropping,
+        Status_Dropped,
+        Status_Aborted,
+        Status_32Bit_Hack = 0x7fffffff
     };
 
 public:
@@ -70,26 +74,37 @@ public:
 
     int Abort(void);
     void SetStatus(Status status);
-    int Signal(const RTCString &strFormat, const void *pvData, uint32_t cbData);
+    int Signal(const RTCString &strFormat, const void *pvData, size_t cbData);
 
 protected:
 
     bool LookupFormatEtc(LPFORMATETC pFormatEtc, ULONG *puIndex);
-    static HGLOBAL MemDup(HGLOBAL hMemSource);
     void RegisterFormat(LPFORMATETC pFormatEtc, CLIPFORMAT clipFormat, TYMED tyMed = TYMED_HGLOBAL,
                         LONG lindex = -1, DWORD dwAspect = DVASPECT_CONTENT, DVTARGETDEVICE *pTargetDevice = NULL);
 
-    Status      mStatus;
-    LONG        mRefCount;
-    ULONG       mcFormats;
-    LPFORMATETC mpFormatEtc;
-    LPSTGMEDIUM mpStgMedium;
-    RTSEMEVENT  mEventDropped;
-    RTCString   mstrFormat;
-    void       *mpvData;
-    uint32_t    mcbData;
+    /** Current drag and drop status. */
+    Status      m_enmStatus;
+    /** Internal reference count of this object. */
+    LONG        m_cRefs;
+    /** Number of native formats registered. This can be a different number than supplied with m_lstFormats. */
+    ULONG       m_cFormats;
+    /** Array of registered FORMATETC structs. Matches m_cFormats. */
+    LPFORMATETC m_paFormatEtc;
+    /** Array of registered STGMEDIUM structs. Matches m_cFormats. */
+    LPSTGMEDIUM m_paStgMedium;
+    /** Event semaphore used for waiting on status changes. */
+    RTSEMEVENT  m_EvtDropped;
+    /** Format of currently retrieved data. */
+    RTCString   m_strFormat;
+    /** The retrieved data as a raw buffer. */
+    void       *m_pvData;
+    /** Raw buffer size (in bytes). */
+    size_t      m_cbData;
 };
 
+/**
+ * Class for implementing IDropSource for VBoxTray's DnD support.
+ */
 class VBoxDnDDropSource : public IDropSource
 {
 public:
@@ -99,7 +114,7 @@ public:
 
 public:
 
-    VBOXDNDACTION GetCurrentAction(void) { return mDnDActionCurrent; }
+    VBOXDNDACTION GetCurrentAction(void) { return m_enmActionCurrent; }
 
 public: /* IUnknown methods. */
 
@@ -115,15 +130,18 @@ public: /* IDropSource methods. */
 protected:
 
     /** Reference count of this object. */
-    LONG                  mRefCount;
+    LONG                  m_cRefs;
     /** Pointer to parent proxy window. */
-    VBoxDnDWnd           *mpWndParent;
+    VBoxDnDWnd           *m_pWndParent;
     /** Current drag effect. */
-    DWORD                 mdwCurEffect;
+    DWORD                 m_dwCurEffect;
     /** Current action to perform on the host. */
-    VBOXDNDACTION         mDnDActionCurrent;
+    VBOXDNDACTION         m_enmActionCurrent;
 };
 
+/**
+ * Class for implementing IDropTarget for VBoxTray's DnD support.
+ */
 class VBoxDnDDropTarget : public IDropTarget
 {
 public:
@@ -152,34 +170,41 @@ protected:
 
 public:
 
-    void *DataMutableRaw(void) const { return mpvData; }
-    size_t DataSize(void) const { return mcbData; }
+    /** Returns the data as mutable raw. Use with caution! */
+    void *DataMutableRaw(void) const { return m_pvData; }
+
+    /** Returns the data size (in bytes). */
+    size_t DataSize(void) const { return m_cbData; }
+
     RTCString Formats(void) const;
     int WaitForDrop(RTMSINTERVAL msTimeout);
 
 protected:
 
     /** Reference count of this object. */
-    LONG                  mRefCount;
+    LONG                  m_cRefs;
     /** Pointer to parent proxy window. */
-    VBoxDnDWnd           *mpWndParent;
+    VBoxDnDWnd           *m_pWndParent;
     /** Current drop effect. */
-    DWORD                 mdwCurEffect;
+    DWORD                 m_dwCurEffect;
     /** Copy of the data object's current FORMATETC struct.
      *  Note: We don't keep the pointer of the DVTARGETDEVICE here! */
-    FORMATETC             mFormatEtc;
-    /** Stringified data object's formats string.  */
-    RTCString             mstrFormats;
+    FORMATETC             m_FormatEtc;
+    /** Stringified data object's format currently in use.  */
+    RTCString             m_strFormat;
     /** Pointer to actual format data. */
-    void                 *mpvData;
+    void                 *m_pvData;
     /** Size (in bytes) of format data. */
-    size_t                mcbData;
+    size_t                m_cbData;
     /** Event for waiting on the "drop" event. */
-    RTSEMEVENT            hEventDrop;
+    RTSEMEVENT            m_EvtDrop;
     /** Result of the drop event. */
-    int                   mDroppedRc;
+    int                   m_rcDropped;
 };
 
+/**
+ * Class for implementing IEnumFORMATETC for VBoxTray's DnD support.
+ */
 class VBoxDnDEnumFormatEtc : public IEnumFORMATETC
 {
 public:
@@ -205,16 +230,20 @@ public:
 
 private:
 
-    LONG        m_lRefCount;
-    ULONG       m_nIndex;
-    ULONG       m_nNumFormats;
-    LPFORMATETC m_pFormatEtc;
+    /** Reference count of this object. */
+    LONG        m_cRefs;
+    /** Current index for format iteration. */
+    ULONG       m_uIdxCur;
+    /** Number of format this object contains. */
+    ULONG       m_cFormats;
+    /** Array of FORMATETC formats this object contains. Matches m_cFormats. */
+    LPFORMATETC m_paFormatEtc;
 };
 
 struct VBOXDNDCONTEXT;
 class VBoxDnDWnd;
 
-/*
+/**
  * A drag'n drop event from the host.
  */
 typedef struct VBOXDNDEVENT
@@ -307,14 +336,13 @@ public:
 
 public:
 
-    int Initialize(PVBOXDNDCONTEXT pContext);
+    int Initialize(PVBOXDNDCONTEXT a_pCtx);
     void Destroy(void);
 
 public:
 
-    /** The window's thread for the native message pump and
-     *  OLE context. */
-    static int Thread(RTTHREAD hThread, void *pvUser);
+    /** The window's thread for the native message pump and OLE context. */
+    static DECLCALLBACK(int) Thread(RTTHREAD hThread, void *pvUser);
 
 public:
 
@@ -337,7 +365,7 @@ public:
     int Abort(void);
 
     /* Host -> Guest */
-    int OnHgEnter(const RTCList<RTCString> &formats, VBOXDNDACTIONLIST dndLstActionsAllowed);
+    int OnHgEnter(const RTCList<RTCString> &formats, VBOXDNDACTIONLIST m_lstActionsAllowed);
     int OnHgMove(uint32_t u32xPos, uint32_t u32yPos, VBOXDNDACTION dndAction);
     int OnHgDrop(void);
     int OnHgLeave(void);
@@ -367,48 +395,50 @@ protected:
 public: /** @todo Make protected! */
 
     /** Pointer to DnD context. */
-    PVBOXDNDCONTEXT            pCtx;
+    PVBOXDNDCONTEXT            m_pCtx;
     /** The proxy window's main thread for processing
      *  window messages. */
-    RTTHREAD                   hThread;
-    RTCRITSECT                 mCritSect;
-    RTSEMEVENT                 mEventSem;
+    RTTHREAD                   m_hThread;
+    /** Critical section to serialize access. */
+    RTCRITSECT                 m_CritSect;
+    /** Event semaphore to wait for new DnD events. */
+    RTSEMEVENT                 m_EvtSem;
 #ifdef RT_OS_WINDOWS
     /** The window's handle. */
-    HWND                       hWnd;
+    HWND                       m_hWnd;
     /** List of allowed MIME types this
      *  client can handle. Make this a per-instance
      *  property so that we can selectively allow/forbid
      *  certain types later on runtime. */
-    RTCList<RTCString>         lstFmtSup;
+    RTCList<RTCString>         m_lstFmtSup;
     /** List of formats for the current
      *  drag'n drop operation. */
-    RTCList<RTCString>         lstFmtActive;
+    RTCList<RTCString>         m_lstFmtActive;
     /** List of all current drag'n drop actions allowed. */
-    VBOXDNDACTIONLIST          dndLstActionsAllowed;
+    VBOXDNDACTIONLIST          m_lstActionsAllowed;
     /** The startup information required
      *  for the actual DoDragDrop() call. */
-    VBOXDNDSTARTUPINFO         startupInfo;
+    VBOXDNDSTARTUPINFO         m_startupInfo;
     /** Is the left mouse button being pressed
      *  currently while being in this window? */
-    bool                       mfMouseButtonDown;
+    bool                       m_fMouseButtonDown;
 # ifdef VBOX_WITH_DRAG_AND_DROP_GH
-    /** IDropTarget implementation for guest -> host
-     *  support. */
-    VBoxDnDDropTarget         *pDropTarget;
+    /** Pointer to IDropTarget implementation for
+     *  guest -> host support. */
+    VBoxDnDDropTarget         *m_pDropTarget;
 # endif /* VBOX_WITH_DRAG_AND_DROP_GH */
 #else /* !RT_OS_WINDOWS */
     /** @todo Implement me. */
 #endif /* !RT_OS_WINDOWS */
 
     /** The window's own DnD context. */
-    VBGLR3GUESTDNDCMDCTX       mDnDCtx;
+    VBGLR3GUESTDNDCMDCTX       m_cmdCtx;
     /** The current operation mode. */
-    Mode                       mMode;
+    Mode                       m_enmMode;
     /** The current state. */
-    State                      mState;
+    State                      m_enmState;
     /** Format being requested. */
-    RTCString                  mFormatRequested;
+    RTCString                  m_strFmtReq;
 };
 
 #endif /* !GA_INCLUDED_SRC_WINNT_VBoxTray_VBoxDnD_h */
