@@ -41,11 +41,11 @@
 /** @todo Implement IDataObjectAsyncCapability interface? */
 
 VBoxDnDDataObject::VBoxDnDDataObject(LPFORMATETC pFormatEtc, LPSTGMEDIUM pStgMed, ULONG cFormats)
-    : mStatus(Uninitialized),
-      mRefCount(1),
-      mcFormats(0),
-      mpvData(NULL),
-      mcbData(0)
+    : m_enmStatus(Status_Uninitialized),
+      m_cRefs(1),
+      m_cFormats(0),
+      m_pvData(NULL),
+      m_cbData(0)
 {
     HRESULT hr;
 
@@ -54,10 +54,10 @@ VBoxDnDDataObject::VBoxDnDDataObject(LPFORMATETC pFormatEtc, LPSTGMEDIUM pStgMed
 
     try
     {
-        mpFormatEtc = new FORMATETC[cAllFormats];
-        RT_BZERO(mpFormatEtc, sizeof(FORMATETC) * cAllFormats);
-        mpStgMedium = new STGMEDIUM[cAllFormats];
-        RT_BZERO(mpStgMedium, sizeof(STGMEDIUM) * cAllFormats);
+        m_paFormatEtc = new FORMATETC[cAllFormats];
+        RT_BZERO(m_paFormatEtc, sizeof(FORMATETC) * cAllFormats);
+        m_paStgMedium = new STGMEDIUM[cAllFormats];
+        RT_BZERO(m_paStgMedium, sizeof(STGMEDIUM) * cAllFormats);
 
         /*
          * Registration of dynamic formats needed?
@@ -72,8 +72,8 @@ VBoxDnDDataObject::VBoxDnDDataObject(LPFORMATETC pFormatEtc, LPSTGMEDIUM pStgMed
             {
                 LogFlowFunc(("Format %RU32: cfFormat=%RI16, tyMed=%RU32, dwAspect=%RU32\n",
                              i, pFormatEtc[i].cfFormat, pFormatEtc[i].tymed, pFormatEtc[i].dwAspect));
-                mpFormatEtc[i] = pFormatEtc[i];
-                mpStgMedium[i] = pStgMed[i];
+                m_paFormatEtc[i] = pFormatEtc[i];
+                m_paStgMedium[i] = pStgMed[i];
             }
         }
 
@@ -86,7 +86,7 @@ VBoxDnDDataObject::VBoxDnDDataObject(LPFORMATETC pFormatEtc, LPSTGMEDIUM pStgMed
 
     if (SUCCEEDED(hr))
     {
-        int rc2 = RTSemEventCreate(&mEventDropped);
+        int rc2 = RTSemEventCreate(&m_EvtDropped);
         AssertRC(rc2);
 
         /*
@@ -114,8 +114,8 @@ VBoxDnDDataObject::VBoxDnDDataObject(LPFORMATETC pFormatEtc, LPSTGMEDIUM pStgMed
         RegisterFormat(&mpFormatEtc[cFormats++],
                        RegisterClipboardFormat(CFSTR_SHELLIDLISTOFFSET));
 #endif
-        mcFormats = cFormats;
-        mStatus   = Initialized;
+        m_cFormats = cFormats;
+        m_enmStatus   = Status_Initialized;
     }
 
     LogFlowFunc(("cFormats=%RU32, hr=%Rhrc\n", cFormats, hr));
@@ -123,16 +123,16 @@ VBoxDnDDataObject::VBoxDnDDataObject(LPFORMATETC pFormatEtc, LPSTGMEDIUM pStgMed
 
 VBoxDnDDataObject::~VBoxDnDDataObject(void)
 {
-    if (mpFormatEtc)
-        delete[] mpFormatEtc;
+    if (m_paFormatEtc)
+        delete[] m_paFormatEtc;
 
-    if (mpStgMedium)
-        delete[] mpStgMedium;
+    if (m_paStgMedium)
+        delete[] m_paStgMedium;
 
-    if (mpvData)
-        RTMemFree(mpvData);
+    if (m_pvData)
+        RTMemFree(m_pvData);
 
-    LogFlowFunc(("mRefCount=%RI32\n", mRefCount));
+    LogFlowFunc(("mRefCount=%RI32\n", m_cRefs));
 }
 
 /*
@@ -141,12 +141,12 @@ VBoxDnDDataObject::~VBoxDnDDataObject(void)
 
 STDMETHODIMP_(ULONG) VBoxDnDDataObject::AddRef(void)
 {
-    return InterlockedIncrement(&mRefCount);
+    return InterlockedIncrement(&m_cRefs);
 }
 
 STDMETHODIMP_(ULONG) VBoxDnDDataObject::Release(void)
 {
-    LONG lCount = InterlockedDecrement(&mRefCount);
+    LONG lCount = InterlockedDecrement(&m_cRefs);
     if (lCount == 0)
     {
         delete this;
@@ -172,15 +172,6 @@ STDMETHODIMP VBoxDnDDataObject::QueryInterface(REFIID iid, void **ppvObject)
     return E_NOINTERFACE;
 }
 
-/**
- * Retrieves the data stored in this object and store the result in
- * pMedium.
- *
- * @return  IPRT status code.
- * @return  HRESULT
- * @param   pFormatEtc
- * @param   pMedium
- */
 STDMETHODIMP VBoxDnDDataObject::GetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMedium)
 {
     AssertPtrReturn(pFormatEtc, DV_E_FORMATETC);
@@ -189,35 +180,35 @@ STDMETHODIMP VBoxDnDDataObject::GetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMed
     ULONG lIndex;
     if (!LookupFormatEtc(pFormatEtc, &lIndex)) /* Format supported? */
         return DV_E_FORMATETC;
-    if (lIndex >= mcFormats) /* Paranoia. */
+    if (lIndex >= m_cFormats) /* Paranoia. */
         return DV_E_FORMATETC;
 
-    LPFORMATETC pThisFormat = &mpFormatEtc[lIndex];
+    LPFORMATETC pThisFormat = &m_paFormatEtc[lIndex];
     AssertPtr(pThisFormat);
 
-    LPSTGMEDIUM pThisMedium = &mpStgMedium[lIndex];
+    LPSTGMEDIUM pThisMedium = &m_paStgMedium[lIndex];
     AssertPtr(pThisMedium);
 
     LogFlowFunc(("Using pThisFormat=%p, pThisMedium=%p\n", pThisFormat, pThisMedium));
 
     HRESULT hr = DV_E_FORMATETC; /* Play safe. */
 
-    LogFlowFunc(("mStatus=%ld\n", mStatus));
-    if (mStatus == Dropping)
+    LogFlowFunc(("mStatus=%ld\n", m_enmStatus));
+    if (m_enmStatus == Status_Dropping)
     {
         LogRel2(("DnD: Waiting for drop event ...\n"));
-        int rc2 = RTSemEventWait(mEventDropped, RT_INDEFINITE_WAIT);
-        LogFlowFunc(("rc2=%Rrc, mStatus=%ld\n", rc2, mStatus)); RT_NOREF(rc2);
+        int rc2 = RTSemEventWait(m_EvtDropped, RT_INDEFINITE_WAIT);
+        LogFlowFunc(("rc2=%Rrc, mStatus=%ld\n", rc2, m_enmStatus)); RT_NOREF(rc2);
     }
 
-    if (mStatus == Dropped)
+    if (m_enmStatus == Status_Dropped)
     {
         LogRel2(("DnD: Drop event received\n"));
         LogRel3(("DnD: cfFormat=%RI16, sFormat=%s, tyMed=%RU32, dwAspect=%RU32\n",
                  pThisFormat->cfFormat, VBoxDnDDataObject::ClipboardFormatToString(pFormatEtc->cfFormat),
                  pThisFormat->tymed, pThisFormat->dwAspect));
         LogRel3(("DnD: Got strFormat=%s, pvData=%p, cbData=%RU32\n",
-                  mstrFormat.c_str(), mpvData, mcbData));
+                  m_strFormat.c_str(), m_pvData, m_cbData));
 
         /*
          * Initialize default values.
@@ -228,37 +219,17 @@ STDMETHODIMP VBoxDnDDataObject::GetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMed
         /*
          * URI list handling.
          */
-        if (mstrFormat.equalsIgnoreCase("text/uri-list"))
+        if (DnDMIMEHasFileURLs(m_strFormat.c_str(), RTSTR_MAX))
         {
-            int rc = VINF_SUCCESS;
-
-            RTCList<RTCString> lstFilesURI = RTCString((char*)mpvData, mcbData).split("\r\n");
-            RTCList<RTCString> lstFiles;
-            for (size_t i = 0; i < lstFilesURI.size(); i++)
-            {
-                char *pszFilePath = RTUriFilePath(lstFilesURI.at(i).c_str());
-                if (pszFilePath)
-                {
-                    lstFiles.append(pszFilePath);
-                    RTStrFree(pszFilePath);
-                }
-                else /* Unable to parse -- refuse entire request. */
-                {
-                    lstFiles.clear();
-                    rc = VERR_INVALID_PARAMETER;
-                    break;
-                }
-            }
-
-            size_t cFiles = lstFiles.size();
+            char **papszFiles;
+            size_t cFiles;
+            int rc = RTStrSplit((const char *)m_pvData, m_cbData, DND_PATH_SEPARATOR_STR, &papszFiles, &cFiles);
             if (   RT_SUCCESS(rc)
                 && cFiles)
             {
-#ifdef DEBUG
-                LogFlowFunc(("Files (%zu)\n", cFiles));
+                LogRel2(("DnD: Files (%zu)\n", cFiles));
                 for (size_t i = 0; i < cFiles; i++)
-                    LogFlowFunc(("\tFile: %s\n", lstFiles.at(i).c_str()));
-#endif
+                    LogRel2(("\tDnD: File '%s'\n", papszFiles[i]));
 
 #if 0
                 if (   (pFormatEtc->tymed & TYMED_ISTREAM)
@@ -291,12 +262,12 @@ STDMETHODIMP VBoxDnDDataObject::GetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMed
                     && (pFormatEtc->dwAspect == DVASPECT_CONTENT)
                     && (pFormatEtc->cfFormat == CF_TEXT))
                 {
-                    pMedium->hGlobal = GlobalAlloc(GHND, mcbData + 1);
+                    pMedium->hGlobal = GlobalAlloc(GHND, m_cbData + 1);
                     if (pMedium->hGlobal)
                     {
                         char *pcDst  = (char *)GlobalLock(pMedium->hGlobal);
-                        memcpy(pcDst, mpvData, mcbData);
-                        pcDst[mcbData] = '\0';
+                        memcpy(pcDst, m_pvData, m_cbData);
+                        pcDst[m_cbData] = '\0';
                         GlobalUnlock(pMedium->hGlobal);
 
                         hr = S_OK;
@@ -309,7 +280,7 @@ STDMETHODIMP VBoxDnDDataObject::GetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMed
                     size_t cchFiles = 0; /* Number of ASCII characters. */
                     for (size_t i = 0; i < cFiles; i++)
                     {
-                        cchFiles += strlen(lstFiles.at(i).c_str());
+                        cchFiles += strlen(papszFiles[i]);
                         cchFiles += 1; /* Terminating '\0'. */
                     }
 
@@ -327,7 +298,7 @@ STDMETHODIMP VBoxDnDDataObject::GetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMed
                         {
                             size_t cchCurFile;
                             PRTUTF16 pwszFile;
-                            rc = RTStrToUtf16(lstFiles.at(i).c_str(), &pwszFile);
+                            rc = RTStrToUtf16(papszFiles[i], &pwszFile);
                             if (RT_SUCCESS(rc))
                             {
                                 cchCurFile = RTUtf16Len(pwszFile);
@@ -372,6 +343,10 @@ STDMETHODIMP VBoxDnDDataObject::GetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMed
                     else
                         rc = VERR_NO_MEMORY;
                 }
+
+                for (size_t i = 0; i < cFiles; ++i)
+                    RTStrFree(papszFiles[i]);
+                RTMemFree(papszFiles);
             }
 
             if (RT_FAILURE(rc))
@@ -380,28 +355,28 @@ STDMETHODIMP VBoxDnDDataObject::GetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMed
         /*
          * Plain text handling.
          */
-        else if (   mstrFormat.equalsIgnoreCase("text/plain")
-                 || mstrFormat.equalsIgnoreCase("text/html")
-                 || mstrFormat.equalsIgnoreCase("text/plain;charset=utf-8")
-                 || mstrFormat.equalsIgnoreCase("text/plain;charset=utf-16")
-                 || mstrFormat.equalsIgnoreCase("text/richtext")
-                 || mstrFormat.equalsIgnoreCase("UTF8_STRING")
-                 || mstrFormat.equalsIgnoreCase("TEXT")
-                 || mstrFormat.equalsIgnoreCase("STRING"))
+        else if (   m_strFormat.equalsIgnoreCase("text/plain")
+                 || m_strFormat.equalsIgnoreCase("text/html")
+                 || m_strFormat.equalsIgnoreCase("text/plain;charset=utf-8")
+                 || m_strFormat.equalsIgnoreCase("text/plain;charset=utf-16")
+                 || m_strFormat.equalsIgnoreCase("text/richtext")
+                 || m_strFormat.equalsIgnoreCase("UTF8_STRING")
+                 || m_strFormat.equalsIgnoreCase("TEXT")
+                 || m_strFormat.equalsIgnoreCase("STRING"))
         {
-            pMedium->hGlobal = GlobalAlloc(GHND, mcbData + 1);
+            pMedium->hGlobal = GlobalAlloc(GHND, m_cbData + 1);
             if (pMedium->hGlobal)
             {
                 char *pcDst  = (char *)GlobalLock(pMedium->hGlobal);
-                memcpy(pcDst, mpvData, mcbData);
-                pcDst[mcbData] = '\0';
+                memcpy(pcDst, m_pvData, m_cbData);
+                pcDst[m_cbData] = '\0';
                 GlobalUnlock(pMedium->hGlobal);
 
                 hr = S_OK;
             }
         }
         else
-            LogRel(("DnD: Error: Format '%s' not implemented\n", mstrFormat.c_str()));
+            LogRel(("DnD: Error: Format '%s' not implemented\n", m_strFormat.c_str()));
     }
 
     /* Error handling; at least return some basic data. */
@@ -425,20 +400,12 @@ STDMETHODIMP VBoxDnDDataObject::GetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMed
     }
 
     if (hr == DV_E_FORMATETC)
-        LogRel(("DnD: Error handling format '%s' (%RU32 bytes)\n", mstrFormat.c_str(), mcbData));
+        LogRel(("DnD: Error handling format '%s' (%RU32 bytes)\n", m_strFormat.c_str(), m_cbData));
 
     LogFlowFunc(("hr=%Rhrc\n", hr));
     return hr;
 }
 
-/**
- * Only required for IStream / IStorage interfaces.
- *
- * @return  IPRT status code.
- * @return  HRESULT
- * @param   pFormatEtc
- * @param   pMedium
- */
 STDMETHODIMP VBoxDnDDataObject::GetDataHere(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMedium)
 {
     RT_NOREF(pFormatEtc, pMedium);
@@ -446,13 +413,6 @@ STDMETHODIMP VBoxDnDDataObject::GetDataHere(LPFORMATETC pFormatEtc, LPSTGMEDIUM
     return DATA_E_FORMATETC;
 }
 
-/**
- * Query if this objects supports a specific format.
- *
- * @return  IPRT status code.
- * @return  HRESULT
- * @param   pFormatEtc
- */
 STDMETHODIMP VBoxDnDDataObject::QueryGetData(LPFORMATETC pFormatEtc)
 {
     LogFlowFunc(("\n"));
@@ -477,11 +437,11 @@ STDMETHODIMP VBoxDnDDataObject::SetData(LPFORMATETC pFormatEtc, LPSTGMEDIUM pMed
 
 STDMETHODIMP VBoxDnDDataObject::EnumFormatEtc(DWORD dwDirection, IEnumFORMATETC **ppEnumFormatEtc)
 {
-    LogFlowFunc(("dwDirection=%RI32, mcFormats=%RI32, mpFormatEtc=%p\n", dwDirection, mcFormats, mpFormatEtc));
+    LogFlowFunc(("dwDirection=%RI32, mcFormats=%RI32, mpFormatEtc=%p\n", dwDirection, m_cFormats, m_paFormatEtc));
 
     HRESULT hr;
     if (dwDirection == DATADIR_GET)
-        hr = VBoxDnDEnumFormatEtc::CreateEnumFormatEtc(mcFormats, mpFormatEtc, ppEnumFormatEtc);
+        hr = VBoxDnDEnumFormatEtc::CreateEnumFormatEtc(m_cFormats, m_paFormatEtc, ppEnumFormatEtc);
     else
         hr = E_NOTIMPL;
 
@@ -511,13 +471,24 @@ STDMETHODIMP VBoxDnDDataObject::EnumDAdvise(IEnumSTATDATA **ppEnumAdvise)
  * Own stuff.
  */
 
+/**
+ * Aborts waiting for data being "dropped".
+ *
+ * @returns VBox status code.
+ */
 int VBoxDnDDataObject::Abort(void)
 {
     LogFlowFunc(("Aborting ...\n"));
-    mStatus = Aborted;
-    return RTSemEventSignal(mEventDropped);
+    m_enmStatus = Status_Aborted;
+    return RTSemEventSignal(m_EvtDropped);
 }
 
+/**
+ * Static helper function to convert a CLIPFORMAT to a string and return it.
+ *
+ * @returns Pointer to converted stringified CLIPFORMAT, or "unknown" if not found / invalid.
+ * @param   fmt                 CLIPFORMAT to return string for.
+ */
 /* static */
 const char* VBoxDnDDataObject::ClipboardFormatToString(CLIPFORMAT fmt)
 {
@@ -601,19 +572,26 @@ const char* VBoxDnDDataObject::ClipboardFormatToString(CLIPFORMAT fmt)
     return "unknown";
 }
 
+/**
+ * Checks whether a given FORMATETC is supported by this data object and returns its index.
+ *
+ * @returns \c true if format is supported, \c false if not.
+ * @param   pFormatEtc          Pointer to FORMATETC to check for.
+ * @param   puIndex             Where to store the index if format is supported.
+ */
 bool VBoxDnDDataObject::LookupFormatEtc(LPFORMATETC pFormatEtc, ULONG *puIndex)
 {
     AssertReturn(pFormatEtc, false);
     /* puIndex is optional. */
 
-    for (ULONG i = 0; i < mcFormats; i++)
+    for (ULONG i = 0; i < m_cFormats; i++)
     {
-        if(    (pFormatEtc->tymed & mpFormatEtc[i].tymed)
-            && pFormatEtc->cfFormat == mpFormatEtc[i].cfFormat
-            && pFormatEtc->dwAspect == mpFormatEtc[i].dwAspect)
+        if(    (pFormatEtc->tymed & m_paFormatEtc[i].tymed)
+            && pFormatEtc->cfFormat == m_paFormatEtc[i].cfFormat
+            && pFormatEtc->dwAspect == m_paFormatEtc[i].dwAspect)
         {
             LogRel3(("DnD: Format found: tyMed=%RI32, cfFormat=%RI16, sFormats=%s, dwAspect=%RI32, ulIndex=%RU32\n",
-                      pFormatEtc->tymed, pFormatEtc->cfFormat, VBoxDnDDataObject::ClipboardFormatToString(mpFormatEtc[i].cfFormat),
+                      pFormatEtc->tymed, pFormatEtc->cfFormat, VBoxDnDDataObject::ClipboardFormatToString(m_paFormatEtc[i].cfFormat),
                       pFormatEtc->dwAspect, i));
             if (puIndex)
                 *puIndex = i;
@@ -628,25 +606,16 @@ bool VBoxDnDDataObject::LookupFormatEtc(LPFORMATETC pFormatEtc, ULONG *puIndex)
     return false;
 }
 
-/* static */
-HGLOBAL VBoxDnDDataObject::MemDup(HGLOBAL hMemSource)
-{
-    DWORD dwLen    = GlobalSize(hMemSource);
-    AssertReturn(dwLen, NULL);
-    PVOID pvSource = GlobalLock(hMemSource);
-    if (pvSource)
-    {
-        PVOID pvDest = GlobalAlloc(GMEM_FIXED, dwLen);
-        if (pvDest)
-            memcpy(pvDest, pvSource, dwLen);
-
-        GlobalUnlock(hMemSource);
-        return pvDest;
-    }
-
-    return NULL;
-}
-
+/**
+ * Registers a new format with this data object.
+ *
+ * @param   pFormatEtc          Where to store the new format into.
+ * @param   clipFormat          Clipboard format to register.
+ * @param   tyMed               Format medium type to register.
+ * @param   lIndex              Format index to register.
+ * @param   dwAspect            Format aspect to register.
+ * @param   pTargetDevice       Format target device to register.
+ */
 void VBoxDnDDataObject::RegisterFormat(LPFORMATETC pFormatEtc, CLIPFORMAT clipFormat,
                                        TYMED tyMed, LONG lIndex, DWORD dwAspect,
                                        DVTARGETDEVICE *pTargetDevice)
@@ -663,24 +632,37 @@ void VBoxDnDDataObject::RegisterFormat(LPFORMATETC pFormatEtc, CLIPFORMAT clipFo
                  pFormatEtc->cfFormat, VBoxDnDDataObject::ClipboardFormatToString(pFormatEtc->cfFormat)));
 }
 
+/**
+ * Sets the current status of this data object.
+ *
+ * @param   status              New status to set.
+ */
 void VBoxDnDDataObject::SetStatus(Status status)
 {
     LogFlowFunc(("Setting status to %ld\n", status));
-    mStatus = status;
+    m_enmStatus = status;
 }
 
+/**
+ * Signals that data has been "dropped".
+ *
+ * @returns VBox status code.
+ * @param   strFormat           Format of data (MIME string).
+ * @param   pvData              Pointer to data.
+ * @param   cbData              Size (in bytes) of data.
+ */
 int VBoxDnDDataObject::Signal(const RTCString &strFormat,
-                              const void *pvData, uint32_t cbData)
+                              const void *pvData, size_t cbData)
 {
     int rc;
 
     if (cbData)
     {
-        mpvData = RTMemAlloc(cbData);
-        if (mpvData)
+        m_pvData = RTMemAlloc(cbData);
+        if (m_pvData)
         {
-            memcpy(mpvData, pvData, cbData);
-            mcbData = cbData;
+            memcpy(m_pvData, pvData, cbData);
+            m_cbData = cbData;
             rc = VINF_SUCCESS;
         }
         else
@@ -691,22 +673,22 @@ int VBoxDnDDataObject::Signal(const RTCString &strFormat,
 
     if (RT_SUCCESS(rc))
     {
-        mStatus    = Dropped;
-        mstrFormat = strFormat;
+        m_enmStatus    = Status_Dropped;
+        m_strFormat = strFormat;
     }
     else
     {
-        mStatus = Aborted;
+        m_enmStatus = Status_Aborted;
     }
 
     /* Signal in any case. */
     LogRel2(("DnD: Signalling drop event\n"));
 
-    int rc2 = RTSemEventSignal(mEventDropped);
+    int rc2 = RTSemEventSignal(m_EvtDropped);
     if (RT_SUCCESS(rc))
         rc = rc2;
 
-    LogFunc(("mStatus=%RU32, rc=%Rrc\n", mStatus, rc));
+    LogFunc(("mStatus=%RU32, rc=%Rrc\n", m_enmStatus, rc));
     return rc;
 }
 
