@@ -2,6 +2,7 @@
  * Copyright © 2000 Compaq Computer Corporation
  * Copyright © 2002 Hewlett-Packard Company
  * Copyright © 2006 Intel Corporation
+ * Copyright © 2008 Red Hat, Inc.
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -43,26 +44,29 @@
 #include "pixmapstr.h"
 #include "extnsionst.h"
 #include "servermd.h"
+#include "rrtransform.h"
 #include <X11/extensions/randr.h>
 #include <X11/extensions/randrproto.h>
-#ifdef RENDER
 #include <X11/extensions/render.h> 	/* we share subpixel order information */
 #include "picturestr.h"
-#endif
 #include <X11/Xfuncproto.h>
 
 /* required for ABI compatibility for now */
 #define RANDR_10_INTERFACE 1
 #define RANDR_12_INTERFACE 1
+#define RANDR_13_INTERFACE 1 /* requires RANDR_12_INTERFACE */
+#define RANDR_GET_CRTC_INTERFACE 1
+
+#define RANDR_INTERFACE_VERSION 0x0103
 
 typedef XID	RRMode;
 typedef XID	RROutput;
 typedef XID	RRCrtc;
 
-extern int	RREventBase, RRErrorBase;
+extern _X_EXPORT int	RREventBase, RRErrorBase;
 
-extern int (*ProcRandrVector[RRNumberRequests])(ClientPtr);
-extern int (*SProcRandrVector[RRNumberRequests])(ClientPtr);
+extern _X_EXPORT int (*ProcRandrVector[RRNumberRequests])(ClientPtr);
+extern _X_EXPORT int (*SProcRandrVector[RRNumberRequests])(ClientPtr);
     
 /*
  * Modeline for a monitor. Name follows directly after this struct
@@ -115,6 +119,12 @@ struct _rrCrtc {
     CARD16	    *gammaBlue;
     CARD16	    *gammaGreen;
     void	    *devPrivate;
+    Bool	    transforms;
+    RRTransformRec  client_pending_transform;
+    RRTransformRec  client_current_transform;
+    PictTransform   transform;
+    struct pict_f_transform f_transform;
+    struct pict_f_transform f_inverse;
 };
 
 struct _rrOutput {
@@ -161,6 +171,9 @@ typedef Bool (*RRCrtcSetProcPtr) (ScreenPtr		pScreen,
 typedef Bool (*RRCrtcSetGammaProcPtr) (ScreenPtr	pScreen,
 				       RRCrtcPtr	crtc);
 
+typedef Bool (*RRCrtcGetGammaProcPtr) (ScreenPtr	pScreen,
+				       RRCrtcPtr	crtc);
+
 typedef Bool (*RROutputSetPropertyProcPtr) (ScreenPtr		pScreen,
 					    RROutputPtr		output,
 					    Atom		property,
@@ -175,6 +188,23 @@ typedef void (*RRModeDestroyProcPtr) (ScreenPtr	    pScreen,
 
 #endif
 
+#if RANDR_13_INTERFACE
+typedef Bool (*RROutputGetPropertyProcPtr) (ScreenPtr		pScreen,
+					    RROutputPtr		output,
+					    Atom		property);
+typedef Bool (*RRGetPanningProcPtr)    (ScreenPtr		pScrn,
+					RRCrtcPtr		crtc,
+					BoxPtr		totalArea,
+					BoxPtr		trackingArea,
+					INT16		*border);
+typedef Bool (*RRSetPanningProcPtr)    (ScreenPtr		pScrn,
+					RRCrtcPtr		crtc,
+					BoxPtr		totalArea,
+					BoxPtr		trackingArea,
+					INT16		*border);
+
+#endif /* RANDR_13_INTERFACE */
+
 typedef Bool (*RRGetInfoProcPtr) (ScreenPtr pScreen, Rotation *rotations);
 typedef Bool (*RRCloseScreenProcPtr) ( int i, ScreenPtr pscreen);
 
@@ -216,10 +246,16 @@ typedef struct _rrScrPriv {
     RRScreenSetSizeProcPtr  rrScreenSetSize;
     RRCrtcSetProcPtr	    rrCrtcSet;
     RRCrtcSetGammaProcPtr   rrCrtcSetGamma;
+    RRCrtcGetGammaProcPtr   rrCrtcGetGamma;
     RROutputSetPropertyProcPtr	rrOutputSetProperty;
     RROutputValidateModeProcPtr	rrOutputValidateMode;
     RRModeDestroyProcPtr	rrModeDestroy;
 #endif
+#if RANDR_13_INTERFACE
+    RROutputGetPropertyProcPtr	rrOutputGetProperty;
+    RRGetPanningProcPtr	rrGetPanning;
+    RRSetPanningProcPtr	rrSetPanning;
+#endif
     
     /*
      * Private part of the structure; not considered part of the ABI
@@ -239,6 +275,7 @@ typedef struct _rrScrPriv {
 
     int			    numOutputs;
     RROutputPtr		    *outputs;
+    RROutputPtr		    primaryOutput;
 
     int			    numCrtcs;
     RRCrtcPtr		    *crtcs;
@@ -262,7 +299,8 @@ typedef struct _rrScrPriv {
 #endif
 } rrScrPrivRec, *rrScrPrivPtr;
 
-extern DevPrivateKey rrPrivKey;
+extern _X_EXPORT DevPrivateKeyRec rrPrivKeyRec;
+#define rrPrivKey (&rrPrivKeyRec)
 
 #define rrGetScrPriv(pScr)  ((rrScrPrivPtr)dixLookupPrivate(&(pScr)->devPrivates, rrPrivKey))
 #define rrScrPriv(pScr)	rrScrPrivPtr    pScrPriv = rrGetScrPriv(pScr)
@@ -297,32 +335,53 @@ typedef struct _RRClient {
 /*  RRTimesRec	times[0]; */
 } RRClientRec, *RRClientPtr;
 
-extern RESTYPE	RRClientType, RREventType; /* resource types for event masks */
-extern DevPrivateKey RRClientPrivateKey;
-extern RESTYPE	RRCrtcType, RRModeType, RROutputType;
+extern _X_EXPORT RESTYPE RRClientType, RREventType; /* resource types for event masks */
+extern _X_EXPORT DevPrivateKeyRec RRClientPrivateKeyRec;
+#define RRClientPrivateKey (&RRClientPrivateKeyRec)
+extern _X_EXPORT RESTYPE RRCrtcType, RRModeType, RROutputType;
 
-#define LookupOutput(client,id,a) ((RROutputPtr) \
-				   (SecurityLookupIDByType (client, id, \
-							    RROutputType, a)))
-#define LookupCrtc(client,id,a) ((RRCrtcPtr) \
-				 (SecurityLookupIDByType (client, id, \
-							  RRCrtcType, a)))
-#define LookupMode(client,id,a) ((RRModePtr) \
-				 (SecurityLookupIDByType (client, id, \
-							  RRModeType, a)))
+#define VERIFY_RR_OUTPUT(id, ptr, a)\
+    {\
+	int rc = dixLookupResourceByType((pointer *)&(ptr), id,\
+	                                 RROutputType, client, a);\
+	if (rc != Success) {\
+	    client->errorValue = id;\
+	    return rc;\
+	}\
+    }
+
+#define VERIFY_RR_CRTC(id, ptr, a)\
+    {\
+	int rc = dixLookupResourceByType((pointer *)&(ptr), id,\
+	                                 RRCrtcType, client, a);\
+	if (rc != Success) {\
+	    client->errorValue = id;\
+	    return rc;\
+	}\
+    }
+
+#define VERIFY_RR_MODE(id, ptr, a)\
+    {\
+	int rc = dixLookupResourceByType((pointer *)&(ptr), id,\
+	                                 RRModeType, client, a);\
+	if (rc != Success) {\
+	    client->errorValue = id;\
+	    return rc;\
+	}\
+    }
 
 #define GetRRClient(pClient)    ((RRClientPtr)dixLookupPrivate(&(pClient)->devPrivates, RRClientPrivateKey))
 #define rrClientPriv(pClient)	RRClientPtr pRRClient = GetRRClient(pClient)
 
 /* Initialize the extension */
-void
+extern _X_EXPORT void
 RRExtensionInit (void);
 
 #ifdef RANDR_12_INTERFACE
 /*
  * Set the range of sizes for the screen
  */
-void
+extern _X_EXPORT void
 RRScreenSetSizeRange (ScreenPtr	pScreen,
 		      CARD16	minWidth,
 		      CARD16	minHeight,
@@ -336,13 +395,13 @@ RRScreenSetSizeRange (ScreenPtr	pScreen,
  * The driver is responsible for calling this whenever it has changed
  * the size of the screen
  */
-void
+extern _X_EXPORT void
 RRScreenSizeNotify (ScreenPtr	pScreen);
 
 /*
  * Request that the screen be resized
  */
-Bool
+extern _X_EXPORT Bool
 RRScreenSizeSet (ScreenPtr  pScreen,
 		 CARD16	    width,
 		 CARD16	    height,
@@ -352,92 +411,60 @@ RRScreenSizeSet (ScreenPtr  pScreen,
 /*
  * Send ConfigureNotify event to root window when 'something' happens
  */
-void
+extern _X_EXPORT void
 RRSendConfigNotify (ScreenPtr pScreen);
     
 /*
  * screen dispatch
  */
-int 
+extern _X_EXPORT int
 ProcRRGetScreenSizeRange (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRSetScreenSize (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRGetScreenResources (ClientPtr client);
 
-int
+extern _X_EXPORT int
+ProcRRGetScreenResourcesCurrent (ClientPtr client);
+
+extern _X_EXPORT int
 ProcRRSetScreenConfig (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRGetScreenInfo (ClientPtr client);
 
 /*
  * Deliver a ScreenNotify event
  */
-void
+extern _X_EXPORT void
 RRDeliverScreenEvent (ClientPtr client, WindowPtr pWin, ScreenPtr pScreen);
     
-/* mirandr.c */
-Bool
-miRandRInit (ScreenPtr pScreen);
-
-Bool
-miRRGetInfo (ScreenPtr pScreen, Rotation *rotations);
-
-Bool
-miRRGetScreenInfo (ScreenPtr pScreen);
-
-Bool
-miRRCrtcSet (ScreenPtr	pScreen,
-	     RRCrtcPtr	crtc,
-	     RRModePtr	mode,
-	     int	x,
-	     int	y,
-	     Rotation	rotation,
-	     int	numOutput,
-	     RROutputPtr *outputs);
-
-Bool
-miRROutputSetProperty (ScreenPtr	    pScreen,
-		       RROutputPtr	    output,
-		       Atom		    property,
-		       RRPropertyValuePtr   value);
-
-Bool
-miRROutputValidateMode (ScreenPtr	    pScreen,
-			RROutputPtr	    output,
-			RRModePtr	    mode);
-
-void
-miRRModeDestroy (ScreenPtr  pScreen,
-		 RRModePtr  mode);
-
 /* randr.c */
 /*
  * Send all pending events
  */
-void
+extern _X_EXPORT void
 RRTellChanged (ScreenPtr pScreen);
 
 /*
  * Poll the driver for changed information
  */
-Bool
-RRGetInfo (ScreenPtr pScreen);
+extern _X_EXPORT Bool
+RRGetInfo (ScreenPtr pScreen, Bool force_query);
 
-Bool RRInit (void);
+extern _X_EXPORT Bool RRInit (void);
 
-Bool RRScreenInit(ScreenPtr pScreen);
+extern _X_EXPORT Bool RRScreenInit(ScreenPtr pScreen);
 
-RROutputPtr
+extern _X_EXPORT RROutputPtr
 RRFirstOutput (ScreenPtr pScreen);
 
-Rotation
+extern _X_EXPORT Rotation
 RRGetRotation (ScreenPtr pScreen);
 
-CARD16
+extern _X_EXPORT CARD16
 RRVerticalRefresh (xRRModeInfo *mode);
 
 #ifdef RANDR_10_INTERFACE					
@@ -450,14 +477,15 @@ RRVerticalRefresh (xRRModeInfo *mode);
  * Then, register the specific size with the screen
  */
 
-RRScreenSizePtr
+extern _X_EXPORT RRScreenSizePtr
 RRRegisterSize (ScreenPtr		pScreen,
 		short			width, 
 		short			height,
 		short			mmWidth,
 		short			mmHeight);
 
-Bool RRRegisterRate (ScreenPtr		pScreen,
+extern _X_EXPORT Bool
+RRRegisterRate (ScreenPtr		pScreen,
 		     RRScreenSizePtr	pSize,
 		     int		rate);
 
@@ -465,23 +493,17 @@ Bool RRRegisterRate (ScreenPtr		pScreen,
  * Finally, set the current configuration of the screen
  */
 
-void
+extern _X_EXPORT void
 RRSetCurrentConfig (ScreenPtr		pScreen,
 		    Rotation		rotation,
 		    int			rate,
 		    RRScreenSizePtr	pSize);
 
-Bool RRScreenInit (ScreenPtr pScreen);
+extern _X_EXPORT Bool RRScreenInit (ScreenPtr pScreen);
 
-Rotation
+extern _X_EXPORT Rotation
 RRGetRotation (ScreenPtr pScreen);
 
-int
-RRSetScreenConfig (ScreenPtr		pScreen,
-		   Rotation		rotation,
-		   int			rate,
-		   RRScreenSizePtr	pSize);
-
 #endif					
 
 /* rrcrtc.c */
@@ -490,41 +512,48 @@ RRSetScreenConfig (ScreenPtr		pScreen,
  * Notify the CRTC of some change; layoutChanged indicates that
  * some position or size element changed
  */
-void
+extern _X_EXPORT void
 RRCrtcChanged (RRCrtcPtr crtc, Bool layoutChanged);
 
 /*
  * Create a CRTC
  */
-RRCrtcPtr
+extern _X_EXPORT RRCrtcPtr
 RRCrtcCreate (ScreenPtr pScreen, void	*devPrivate);
 
 /*
  * Set the allowed rotations on a CRTC
  */
-void
+extern _X_EXPORT void
 RRCrtcSetRotations (RRCrtcPtr crtc, Rotation rotations);
 
+/*
+ * Set whether transforms are allowed on a CRTC
+ */
+extern _X_EXPORT void
+RRCrtcSetTransformSupport (RRCrtcPtr crtc, Bool transforms);
+
 /*
  * Notify the extension that the Crtc has been reconfigured,
  * the driver calls this whenever it has updated the mode
  */
-Bool
+extern _X_EXPORT Bool
 RRCrtcNotify (RRCrtcPtr	    crtc,
 	      RRModePtr	    mode,
 	      int	    x,
 	      int	    y,
 	      Rotation	    rotation,
+	      RRTransformPtr transform,
 	      int	    numOutputs,
 	      RROutputPtr   *outputs);
 
-void
+extern _X_EXPORT void
 RRDeliverCrtcEvent (ClientPtr client, WindowPtr pWin, RRCrtcPtr crtc);
     
 /*
  * Request that the Crtc be reconfigured
  */
-Bool
+extern _X_EXPORT Bool
 RRCrtcSet (RRCrtcPtr    crtc,
 	   RRModePtr	mode,
 	   int		x,
@@ -537,26 +566,34 @@ RRCrtcSet (RRCrtcPtr    crtc,
  * Request that the Crtc gamma be changed
  */
 
-Bool
+extern _X_EXPORT Bool
 RRCrtcGammaSet (RRCrtcPtr   crtc,
 		CARD16	    *red,
 		CARD16	    *green,
 		CARD16	    *blue);
 
+/*
+ * Request current gamma back from the DDX (if possible).
+ * This includes gamma size.
+ */
+ 
+extern _X_EXPORT Bool
+RRCrtcGammaGet(RRCrtcPtr crtc);
+
 /*
  * Notify the extension that the Crtc gamma has been changed
  * The driver calls this whenever it has changed the gamma values
  * in the RRCrtcRec
  */
 
-Bool
+extern _X_EXPORT Bool
 RRCrtcGammaNotify (RRCrtcPtr	crtc);
 
 /*
  * Set the size of the gamma table at server startup time
  */
 
-Bool
+extern _X_EXPORT Bool
 RRCrtcGammaSetSize (RRCrtcPtr	crtc,
 		    int		size);
 
@@ -565,42 +602,106 @@ RRCrtcGammaSetSize (RRCrtcPtr	crtc,
  * taking into account the current mode and rotation
  */
 
-void
+extern _X_EXPORT void
 RRCrtcGetScanoutSize(RRCrtcPtr crtc, int *width, int *height);
 
+/*
+ * Compute the complete transformation matrix including
+ * client-specified transform, rotation/reflection values and the crtc 
+ * offset.
+ *
+ * Return TRUE if the resulting transform is not a simple translation.
+ */
+extern _X_EXPORT Bool
+RRTransformCompute (int			    x,
+		    int			    y,
+		    int			    width,
+		    int			    height,
+		    Rotation		    rotation,
+		    RRTransformPtr	    rr_transform,
+
+		    PictTransformPtr	    transform,
+		    struct pict_f_transform *f_transform,
+		    struct pict_f_transform *f_inverse);
+
+/*
+ * Return crtc transform
+ */
+extern _X_EXPORT RRTransformPtr
+RRCrtcGetTransform (RRCrtcPtr crtc);
+
+/*
+ * Check whether the pending and current transforms are the same
+ */
+extern _X_EXPORT Bool
+RRCrtcPendingTransform (RRCrtcPtr crtc);
+
 /*
  * Destroy a Crtc at shutdown
  */
-void
+extern _X_EXPORT void
 RRCrtcDestroy (RRCrtcPtr crtc);
 
+
+/*
+ * Set the pending CRTC transformation
+ */
+
+extern _X_EXPORT int
+RRCrtcTransformSet (RRCrtcPtr		crtc,
+		    PictTransformPtr	transform,
+		    struct pict_f_transform *f_transform,
+		    struct pict_f_transform *f_inverse,
+		    char		*filter,
+		    int			filter_len,
+		    xFixed		*params,
+		    int			nparams);
+
 /*
  * Initialize crtc type
  */
-Bool
+extern _X_EXPORT Bool
 RRCrtcInit (void);
 
+/*
+ * Initialize crtc type error value
+ */
+extern _X_EXPORT void
+RRCrtcInitErrorValue (void);
+
 /*
  * Crtc dispatch
  */
 
-int
+extern _X_EXPORT int
 ProcRRGetCrtcInfo (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRSetCrtcConfig (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRGetCrtcGammaSize (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRGetCrtcGamma (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRSetCrtcGamma (ClientPtr client);
 
+extern _X_EXPORT int
+ProcRRSetCrtcTransform (ClientPtr client);
+
+extern _X_EXPORT int
+ProcRRGetCrtcTransform (ClientPtr client);
+
+int
+ProcRRGetPanning (ClientPtr client);
+
+int
+ProcRRSetPanning (ClientPtr client);
+
 /* rrdispatch.c */
-Bool
+extern _X_EXPORT Bool
 RRClientKnowsRates (ClientPtr	pClient);
 
 /* rrmode.c */
@@ -608,42 +709,45 @@ RRClientKnowsRates (ClientPtr	pClient);
  * Find, and if necessary, create a mode
  */
 
-RRModePtr
+extern _X_EXPORT RRModePtr
 RRModeGet (xRRModeInfo	*modeInfo,
 	   const char	*name);
 
-void
-RRModePruneUnused (ScreenPtr pScreen);
-
 /*
  * Destroy a mode.
  */
 
-void
+extern _X_EXPORT void
 RRModeDestroy (RRModePtr mode);
 
 /*
  * Return a list of modes that are valid for some output in pScreen
  */
-RRModePtr *
+extern _X_EXPORT RRModePtr *
 RRModesForScreen (ScreenPtr pScreen, int *num_ret);
     
 /*
  * Initialize mode type
  */
-Bool
+extern _X_EXPORT Bool
 RRModeInit (void);
+
+/*
+ * Initialize mode type error value
+ */
+extern _X_EXPORT void
+RRModeInitErrorValue (void);
     
-int
+extern _X_EXPORT int
 ProcRRCreateMode (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRDestroyMode (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRAddOutputMode (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRDeleteOutputMode (ClientPtr client);
 
 /* rroutput.c */
@@ -654,14 +758,14 @@ ProcRRDeleteOutputMode (ClientPtr client);
  * has changed, or whether the change was strictly internal
  * (which crtc is in use)
  */
-void
+extern _X_EXPORT void
 RROutputChanged (RROutputPtr output, Bool configChanged);
 
 /*
  * Create an output
  */
 
-RROutputPtr
+extern _X_EXPORT RROutputPtr
 RROutputCreate (ScreenPtr   pScreen,
 		const char  *name,
 		int	    nameLength,
@@ -670,112 +774,126 @@ RROutputCreate (ScreenPtr   pScreen,
 /*
  * Notify extension that output parameters have been changed
  */
-Bool
+extern _X_EXPORT Bool
 RROutputSetClones (RROutputPtr  output,
 		   RROutputPtr  *clones,
 		   int		numClones);
 
-Bool
+extern _X_EXPORT Bool
 RROutputSetModes (RROutputPtr	output,
 		  RRModePtr	*modes,
 		  int		numModes,
 		  int		numPreferred);
 
-int
+extern _X_EXPORT int
 RROutputAddUserMode (RROutputPtr    output,
 		     RRModePtr	    mode);
 
-int
+extern _X_EXPORT int
 RROutputDeleteUserMode (RROutputPtr output,
 			RRModePtr   mode);
 
-Bool
+extern _X_EXPORT Bool
 RROutputSetCrtcs (RROutputPtr	output,
 		  RRCrtcPtr	*crtcs,
 		  int		numCrtcs);
 
-Bool
+extern _X_EXPORT Bool
 RROutputSetConnection (RROutputPtr  output,
 		       CARD8	    connection);
 
-Bool
+extern _X_EXPORT Bool
 RROutputSetSubpixelOrder (RROutputPtr output,
 			  int	      subpixelOrder);
 
-Bool
+extern _X_EXPORT Bool
 RROutputSetPhysicalSize (RROutputPtr	output,
 			 int		mmWidth,
 			 int		mmHeight);
 
-void
+extern _X_EXPORT void
 RRDeliverOutputEvent(ClientPtr client, WindowPtr pWin, RROutputPtr output);
 
-void
+extern _X_EXPORT void
 RROutputDestroy (RROutputPtr	output);
 
-int
+extern _X_EXPORT int
 ProcRRGetOutputInfo (ClientPtr client);
 
+extern _X_EXPORT int
+ProcRRSetOutputPrimary (ClientPtr client);
+
+extern _X_EXPORT int
+ProcRRGetOutputPrimary (ClientPtr client);
+
 /*
  * Initialize output type
  */
-Bool
+extern _X_EXPORT Bool
 RROutputInit (void);
+
+/*
+ * Initialize output type error value
+ */
+extern _X_EXPORT void
+RROutputInitErrorValue (void);
     
 /* rrpointer.c */
-void
+extern _X_EXPORT void
 RRPointerMoved (ScreenPtr pScreen, int x, int y);
 
-void
+extern _X_EXPORT void
 RRPointerScreenConfigured (ScreenPtr pScreen);
 
 /* rrproperty.c */
 
-void
+extern _X_EXPORT void
 RRDeleteAllOutputProperties (RROutputPtr output);
 
-RRPropertyValuePtr
+extern _X_EXPORT RRPropertyValuePtr
 RRGetOutputProperty (RROutputPtr output, Atom property, Bool pending);
 
-RRPropertyPtr
+extern _X_EXPORT RRPropertyPtr
 RRQueryOutputProperty (RROutputPtr output, Atom property);
 		       
-void
+extern _X_EXPORT void
 RRDeleteOutputProperty (RROutputPtr output, Atom property);
 
-Bool
+extern _X_EXPORT Bool
 RRPostPendingProperties (RROutputPtr output);
     
-int
+extern _X_EXPORT int
 RRChangeOutputProperty (RROutputPtr output, Atom property, Atom type,
 			int format, int mode, unsigned long len,
 			pointer value, Bool sendevent, Bool pending);
 
-int
+extern _X_EXPORT int
 RRConfigureOutputProperty (RROutputPtr output, Atom property,
 			   Bool pending, Bool range, Bool immutable,
 			   int num_values, INT32 *values);
-int
+extern _X_EXPORT int
 ProcRRChangeOutputProperty (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRGetOutputProperty (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRListOutputProperties (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRQueryOutputProperty (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRConfigureOutputProperty (ClientPtr client);
 
-int
+extern _X_EXPORT int
 ProcRRDeleteOutputProperty (ClientPtr client);
 
 /* rrxinerama.c */
-void
+#ifdef XINERAMA
+extern _X_EXPORT void
 RRXineramaExtensionInit(void);
+#endif
 
 #endif /* _RANDRSTR_H_ */
 
