@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -446,7 +446,7 @@ static int vciFreeImage(PVCICACHE pCache, bool fDelete)
  * @returns VBox status code.
  * @param   cBlocks      The number of blocks the bitmap can manage.
  * @param   ppBlkMap     Where to store the pointer to the block bitmap.
- * @param   pcbBlkMap    Where to store the size of the block bitmap in blocks
+ * @param   pcBlkMap     Where to store the size of the block bitmap in blocks
  *                       needed on the disk.
  */
 static int vciBlkMapCreate(uint64_t cBlocks, PVCIBLKMAP *ppBlkMap, uint32_t *pcBlkMap)
@@ -1425,7 +1425,7 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
 
         /* Setup the root tree. */
         memset(&NodeRoot, 0, sizeof(VciTreeNode));
-        NodeRoot.u8Type = RT_H2LE_U32(VCI_TREE_NODE_TYPE_LEAF);
+        NodeRoot.u8Type = VCI_TREE_NODE_TYPE_LEAF;
 
         rc = vdIfIoIntFileWriteSync(pCache->pIfIo, pCache->pStorage, offTreeRoot,
                                     &NodeRoot, VCI_BYTE2BLOCK(sizeof(VciTreeNode)));
@@ -1455,10 +1455,10 @@ static int vciCreateImage(PVCICACHE pCache, uint64_t cbSize,
 }
 
 /** @copydoc VDCACHEBACKEND::pfnProbe */
-static DECLCALLBACK(int) vciProbe(const char *pszFilename, PVDINTERFACE pVDIfsCache,
+static DECLCALLBACK(int) vciProbe(const char *pszFilename, PVDINTERFACE pVDIfsDisk,
                                   PVDINTERFACE pVDIfsImage)
 {
-    RT_NOREF1(pVDIfsCache);
+    RT_NOREF1(pVDIfsDisk);
     VciHdr Hdr;
     PVDIOSTORAGE pStorage = NULL;
     uint64_t cbFile;
@@ -1982,10 +1982,10 @@ static DECLCALLBACK(void) vciDump(void *pBackendData)
 
 const VDCACHEBACKEND g_VciCacheBackend =
 {
+    /* u32Version */
+    VD_CACHEBACKEND_VERSION,
     /* pszBackendName */
     "vci",
-    /* cbSize */
-    sizeof(VDCACHEBACKEND),
     /* uBackendCaps */
     VD_CAP_CREATE_FIXED | VD_CAP_CREATE_DYNAMIC | VD_CAP_FILE | VD_CAP_VFS,
     /* papszFileExtensions */
@@ -2037,6 +2037,8 @@ const VDCACHEBACKEND g_VciCacheBackend =
     /* pfnComposeLocation */
     NULL,
     /* pfnComposeName */
-    NULL
+    NULL,
+    /* u32VersionEnd */
+    VD_CACHEBACKEND_VERSION
 };
 
